// Generated by CoffeeScript 1.6.3
/*
Renderers
=========

These classes generate output from documentation objects as returned by the
`documentModule` function from the `coffeedoc` module.
*/

var BootstrapRenderer, GithubWikiRenderer, HtmlRenderer, JSONRenderer, Renderer, eco, fs, highlight, marked, path, traverse, _ref, _ref1, _ref2, _ref3,
  __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  __hasProp = {}.hasOwnProperty,
  __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

fs = require('fs');

path = require('path');

eco = require('eco');

traverse = require('traverse');

highlight = require('highlight').Highlight;

marked = require('marked');

Renderer = (function() {
  /*
  Base class for renderers. To create a new renderer, subclass this and
  override the necessary properties and methods.
  */

  Renderer.prototype.indexTemplate = null;

  Renderer.prototype.moduleTemplate = null;

  Renderer.prototype.indexFile = 'index';

  Renderer.prototype.extension = '';

  function Renderer(options) {
    this.render = __bind(this.render, this);
    this.write = __bind(this.write, this);
    this.renderModule = __bind(this.renderModule, this);
    this.renderIndex = __bind(this.renderIndex, this);
    this.options = options;
  }

  Renderer.prototype.renderIndex = function(modules) {
    return eco.render(this.indexTemplate, {
      modules: modules,
      options: this.options
    });
  };

  Renderer.prototype.renderModule = function(module) {
    return eco.render(this.moduleTemplate, {
      module: module,
      options: this.options
    });
  };

  Renderer.prototype._createOutputDir = function(outputdir) {
    /*
    Create output directory, recursively deleting it if it already exists.
    */

    var rm;
    if (path.existsSync(outputdir)) {
      rm = function(target) {
        var p, _i, _len, _ref;
        if (fs.statSync(target).isDirectory()) {
          _ref = fs.readdirSync(target);
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            p = _ref[_i];
            rm(path.join(target, p));
          }
          return fs.rmdirSync(target);
        } else {
          return fs.unlinkSync(target);
        }
      };
      rm(outputdir);
    }
    return fs.mkdirSync(outputdir, '755');
  };

  Renderer.prototype.preprocess = function(module) {
    /*
    Apply any preprocessing to module documentation before it is handed off
    to the template for rendering.
    */

    return module;
  };

  Renderer.prototype.write = function(modules, outputdir) {
    /*
    Render the documentation and generate output for the user. This method
    writes the generated index page to `outputdir`, overwriting the
    directory if it exists. If `outputdir` is not given, prints the index
    page to stdout and exits.
    
    This method does not write the documentation for individual modules.
    Subclasses should override `write` and implement this themselves.
    */

    var index;
    index = this.renderIndex(modules);
    if (!outputdir) {
      process.stdout.write(index);
      process.exit();
    }
    this._createOutputDir(outputdir);
    return fs.writeFile(path.join(outputdir, this.indexFile + this.extension), index);
  };

  Renderer.prototype.render = function(modules, outputdir) {
    var m;
    return this.write((function() {
      var _i, _len, _results;
      _results = [];
      for (_i = 0, _len = modules.length; _i < _len; _i++) {
        m = modules[_i];
        _results.push(this.preprocess(m));
      }
      return _results;
    }).call(this), outputdir);
  };

  return Renderer;

})();

HtmlRenderer = (function(_super) {
  __extends(HtmlRenderer, _super);

  function HtmlRenderer() {
    this.write = __bind(this.write, this);
    this.preprocess = __bind(this.preprocess, this);
    _ref = HtmlRenderer.__super__.constructor.apply(this, arguments);
    return _ref;
  }

  HtmlRenderer.prototype.indexTemplate = fs.readFileSync(__dirname + '/../resources/html/index.eco', 'utf-8');

  HtmlRenderer.prototype.moduleTemplate = fs.readFileSync(__dirname + '/../resources/html/module.eco', 'utf-8');

  HtmlRenderer.prototype.baseCss = fs.readFileSync(__dirname + '/../resources/html/base.css', 'utf-8');

  HtmlRenderer.prototype.extension = '.html';

  HtmlRenderer.prototype.preprocess = function(module) {
    /*
    Convert markdown to html, adding syntax highlighting markup to any code
    blocks.
    
    Add the relative path to the resources directory to the documentation
    object so that generated pages know where to find css.
    */

    var basepath;
    marked.setOptions({
      highlight: function(code, lang) {
        return highlight(code);
      }
    });
    module = traverse(module).map(function(value) {
      if (value && this.key === 'docstring') {
        return this.update(marked(value));
      }
    });
    basepath = path.relative(path.dirname(module.path), process.cwd());
    module.resourcepath = path.join(basepath, 'resources/');
    return module;
  };

  HtmlRenderer.prototype.write = function(modules, outputdir) {
    var dir, dirs, docpath, i, indexPath, length, module, outfile, resourcesdir, _i, _j, _k, _len, _len1,
      _this = this;
    HtmlRenderer.__super__.write.call(this, modules, outputdir);
    for (_i = 0, _len = modules.length; _i < _len; _i++) {
      module = modules[_i];
      docpath = outputdir;
      dirs = path.dirname(path.normalize(module.path)).split('/');
      length = dirs.length;
      indexPath = "index.html";
      if (length > 0 && dirs[0] !== ".") {
        for (i = _j = 0; 0 <= length ? _j < length : _j > length; i = 0 <= length ? ++_j : --_j) {
          indexPath = "../" + indexPath;
        }
      }
      module.indexPath = indexPath;
      for (_k = 0, _len1 = dirs.length; _k < _len1; _k++) {
        dir = dirs[_k];
        docpath = path.join(docpath, dir);
        if (!path.existsSync(docpath)) {
          fs.mkdirSync(docpath, '755');
        }
      }
      outfile = path.join(outputdir, module.path + this.extension);
      fs.writeFile(outfile, this.renderModule(module));
    }
    resourcesdir = path.join(outputdir, 'resources');
    return fs.mkdir(resourcesdir, '755', function() {
      return fs.writeFile(path.join(resourcesdir, 'base.css'), _this.baseCss);
    });
  };

  return HtmlRenderer;

})(Renderer);

GithubWikiRenderer = (function(_super) {
  __extends(GithubWikiRenderer, _super);

  function GithubWikiRenderer() {
    this.write = __bind(this.write, this);
    this.preprocess = __bind(this.preprocess, this);
    _ref1 = GithubWikiRenderer.__super__.constructor.apply(this, arguments);
    return _ref1;
  }

  GithubWikiRenderer.prototype.indexTemplate = fs.readFileSync(__dirname + '/../resources/github-wiki/index.eco', 'utf-8');

  GithubWikiRenderer.prototype.moduleTemplate = fs.readFileSync(__dirname + '/../resources/github-wiki/module.eco', 'utf-8');

  GithubWikiRenderer.prototype.indexFile = 'ModuleIndex';

  GithubWikiRenderer.prototype.extension = '.md';

  GithubWikiRenderer.prototype._wikiize = function(path) {
    var b, bits, bucket, _i, _len;
    bits = path.split('/');
    bucket = [];
    for (_i = 0, _len = bits.length; _i < _len; _i++) {
      b = bits[_i];
      if (b) {
        bucket.push(b[0].toUpperCase() + b.substring(1));
      }
    }
    return bucket.join(':');
  };

  GithubWikiRenderer.prototype._quoteMarkdown = function(t) {
    /*
    It's more than possible that a function name will have underscores...
    quote them.
    */

    return t.replace(/([^\\])?_/g, "$1\\_");
  };

  GithubWikiRenderer.prototype._params = function(t) {
    var a, x, _i, _len;
    a = [];
    for (_i = 0, _len = t.length; _i < _len; _i++) {
      x = t[_i];
      if (x != null) {
        a.push(x);
      } else {
        a.push('{splat}');
      }
    }
    return a.join(', ');
  };

  GithubWikiRenderer.prototype.preprocess = function(module) {
    module.wikiname = this._wikiize(module.path.replace(/\.coffee$/, ''));
    module.quoteMarkdown = this._quoteMarkdown;
    module.params = this._params;
    return module;
  };

  GithubWikiRenderer.prototype.write = function(modules, outputdir) {
    var module, outfile, _i, _len, _results;
    GithubWikiRenderer.__super__.write.call(this, modules, outputdir);
    _results = [];
    for (_i = 0, _len = modules.length; _i < _len; _i++) {
      module = modules[_i];
      outfile = path.join(outputdir, module.wikiname + this.extension);
      _results.push(fs.writeFile(outfile, this.renderModule(module)));
    }
    return _results;
  };

  return GithubWikiRenderer;

})(Renderer);

BootstrapRenderer = (function(_super) {
  __extends(BootstrapRenderer, _super);

  function BootstrapRenderer() {
    this.write = __bind(this.write, this);
    _ref2 = BootstrapRenderer.__super__.constructor.apply(this, arguments);
    return _ref2;
  }

  BootstrapRenderer.prototype.indexTemplate = fs.readFileSync(__dirname + '/../resources/bootstrap-html/index.eco', 'utf-8');

  BootstrapRenderer.prototype.moduleTemplate = fs.readFileSync(__dirname + '/../resources/bootstrap-html/module.eco', 'utf-8');

  BootstrapRenderer.prototype.baseCss = fs.readFileSync(__dirname + '/../resources/bootstrap-html/base.css', 'utf-8');

  BootstrapRenderer.prototype.bootstrapCss = fs.readFileSync(__dirname + '/../resources/bootstrap-html/bootstrap.css', 'utf-8');

  BootstrapRenderer.prototype.extension = '.html';

  BootstrapRenderer.prototype.write = function(modules, outputdir) {
    var resourcesdir,
      _this = this;
    BootstrapRenderer.__super__.write.call(this, modules, outputdir);
    resourcesdir = path.join(outputdir, 'resources');
    return fs.mkdir(resourcesdir, '755', function() {
      return fs.writeFile(path.join(resourcesdir, 'bootstrap.css'), _this.bootstrapCss);
    });
  };

  return BootstrapRenderer;

})(HtmlRenderer);

JSONRenderer = (function(_super) {
  __extends(JSONRenderer, _super);

  function JSONRenderer() {
    _ref3 = JSONRenderer.__super__.constructor.apply(this, arguments);
    return _ref3;
  }

  JSONRenderer.prototype.extension = '.doc.json';

  JSONRenderer.prototype.indexFile = 'index';

  JSONRenderer.prototype.renderIndex = JSON.stringify;

  JSONRenderer.prototype.renderModule = JSON.stringify;

  return JSONRenderer;

})(Renderer);

exports.html = HtmlRenderer;

exports.bootstrap = BootstrapRenderer;

exports.gfm = GithubWikiRenderer;

exports.json = JSONRenderer;
